@@ -2,7 +2,7 @@
 //				Package : omnithread
 // omnithread.h			Created : 7/94 tjr
 //
-//    Copyright (C) 2002-2011 Apasphere Ltd
+//    Copyright (C) 2002-2018 Apasphere Ltd
 //    Copyright (C) 1994-1997 Olivetti & Oracle Research Laboratory
 //
 //    This file is part of the omnithread library
@@ -703,6 +703,10 @@ public:
   int id() { return _id; }
   // return unique thread id within the current process.
 
+  static unsigned long plat_id();
+  // return a platform-specific thread identifier for any thread, not
+  // just ones created with omni_thread.
+
 
   // This class plus the instance of it declared below allows us to execute
   // some initialisation code before main() is called.
