@@ -1,11 +1,7 @@
 #include <echo_callback.hh>
 
-#ifdef HAVE_STD
-#  include <iostream>
-   using namespace std;
-#else
-#  include <iostream.h>
-#endif
+#include <iostream>
+using namespace std;
 
 
 int main(int argc, char** argv)
