@@ -3,7 +3,7 @@
 // httpCrypto.cc              Created on: 2 July 2018
 //                            Author    : Duncan Grisby
 //
-//    Copyright (C) 2018-2019 Apasphere Ltd
+//    Copyright (C) 2018-2024 Apasphere Ltd
 //    Copyright (C) 2018      Apasphere Ltd, BMC Software
 //
 //    This file is part of the omniORB library
@@ -26,6 +26,9 @@
 //    Implementations of the httpCrypto and httpCryptoManager abstract classes.
 //
 
+// This currently uses RSA types and functions that are deprecated in
+// OpenSSL 3.
+
 #include <omniORB4/CORBA.h>
 #include <omniORB4/httpContext.h>
 #include <omniORB4/httpCrypto.h>
@@ -35,7 +38,11 @@
 #include <openssl/rand.h>
 #include <openssl/evp.h>
 #include <openssl/err.h>
-#include <openssl/rsa.h>
+
+#if OPENSSL_VERSION_NUMBER < 0x30000000L
+#  define USE_OLD_RSA_FUNCTIONS
+#  include <openssl/rsa.h>
+#endif
 
 #include <stdio.h>
 #include <string.h>
@@ -64,8 +71,14 @@ namespace {
     std::string key;
     omni_time_t deadline;
   };
-  
-  typedef std::map<std::string, RSA*>       RSAKeyMap;
+
+#ifdef USE_OLD_RSA_FUNCTIONS
+  typedef RSA RSAKey;
+#else
+  typedef EVP_PKEY RSAKey;
+#endif
+
+  typedef std::map<std::string, RSAKey*>    RSAKeyMap;
   typedef std::map<std::string, SessionKey> AESKeyMap;
 };
 
@@ -75,12 +88,12 @@ namespace omni {
 
   class httpCrypto_AES_RSA : public httpCrypto {
   public:
-    
+
     inline httpCrypto_AES_RSA(httpCryptoManager_AES_RSA_impl* impl,
                               const std::string&              peer_ident,
                               const std::string&              self_ident,
-                              RSA*                            self_rsa,
-                              RSA*                            peer_rsa)
+                              RSAKey*                         self_rsa,
+                              RSAKey*                         peer_rsa)
       : pd_impl(impl),
         pd_i_cipher_ctx(0),
         pd_o_cipher_ctx(0),
@@ -98,8 +111,8 @@ namespace omni {
                               const std::string&              key_ident,
                               const std::string&              peer_ident,
                               const std::string&              self_ident,
-                              RSA*                            self_rsa,
-                              RSA*                            peer_rsa,
+                              RSAKey*                         self_rsa,
+                              RSAKey*                         peer_rsa,
                               const omni_time_t&              deadline)
       : pd_impl(impl),
         pd_i_cipher_ctx(0),
@@ -134,7 +147,7 @@ namespace omni {
       // Server side constructor
       memcpy(pd_key, key, KEY_SIZE);
     }
-    
+
     inline ~httpCrypto_AES_RSA()
     {
       if (pd_i_cipher_ctx)
@@ -145,7 +158,7 @@ namespace omni {
     }
 
     virtual const char*    peerIdent();
-    
+
     virtual size_t         writeAuthHeader(char* buf, size_t buf_space);
     virtual CORBA::Boolean matchAuthHeader(const char* val);
     virtual size_t         encryptedSize(size_t giop_size);
@@ -157,7 +170,7 @@ namespace omni {
                            CORBA::Boolean      last);
 
     virtual size_t decryptOverhead();
-  
+
     virtual size_t decrypt(CORBA::Octet*       write_buf,
                            const CORBA::Octet* read_buf,
                            size_t              read_size,
@@ -171,20 +184,20 @@ namespace omni {
     std::string                     pd_key_ident;
     std::string                     pd_peer_ident;
     std::string                     pd_self_ident;
-    RSA*                            pd_self_rsa;
-    RSA*                            pd_peer_rsa;
+    RSAKey*                         pd_self_rsa;
+    RSAKey*                         pd_peer_rsa;
     omni_time_t                     pd_deadline;
     CORBA::Boolean                  pd_key_set;
   };
 
 
   class httpKeyScavenger;
-  
+
   class httpCryptoManager_AES_RSA_impl {
   public:
     httpCryptoManager_AES_RSA_impl();
     ~httpCryptoManager_AES_RSA_impl();
-    
+
     void
     init(const char*    ident,
          const char*    private_key,
@@ -221,7 +234,7 @@ namespace omni {
 
     void
     scavenge();
-    
+
     inline CORBA::ULong
     key_lifetime()
     {
@@ -233,7 +246,7 @@ namespace omni {
     CORBA::ULong      pd_key_lifetime;
 
     std::string       pd_self_ident;
-    RSA*              pd_self_rsa;
+    RSAKey*           pd_self_rsa;
 
     RSAKeyMap         pd_clients;      // ident string -> public key
     RSAKeyMap         pd_servers;      // URL -> public key
@@ -243,7 +256,7 @@ namespace omni {
     omni_tracedmutex  pd_lock;
   };
 
-  
+
   class httpKeyScavenger : public omni_thread {
   public:
     inline httpKeyScavenger(httpCryptoManager_AES_RSA_impl* impl,
@@ -257,9 +270,9 @@ namespace omni {
     void* run_undetached(void*)
     {
       omniORB::logs(25, "httpCrypto key scavenger execute.");
-      
+
       omni_tracedmutex_lock l(pd_lock);
-      
+
       while (pd_go) {
         pd_impl->scavenge();
 
@@ -289,7 +302,7 @@ namespace omni {
     omni_tracedcondition            pd_cond;
   };
 
-  
+
 };
 
 using namespace omni;
@@ -301,6 +314,146 @@ logError(const char* msg, size_t len, void*)
   return 0;
 }
 
+#ifdef USE_OLD_RSA_FUNCTIONS
+
+// Old OpenSSL versions have convenient RSA functions...
+
+static inline void
+privateEncrypt(CORBA::OctetSeq& dest, RSAKey* rsa,
+               const CORBA::Octet* data, size_t len)
+{
+  dest.length(RSA_size(rsa));
+  if (RSA_private_encrypt(len, data, dest.NP_data(),
+                          rsa, RSA_PKCS1_PADDING) == -1) {
+
+    ERR_print_errors_cb(logError, 0);
+    OMNIORB_THROW(MARSHAL, MARSHAL_InvalidEncryptedData, CORBA::COMPLETED_NO);
+  }
+}
+
+static inline void
+publicEncrypt(CORBA::OctetSeq& dest, RSAKey* rsa,
+              const CORBA::Octet* data, size_t len)
+{
+  dest.length(RSA_size(rsa));
+  if (RSA_public_encrypt(len, data, dest.NP_data(),
+                         rsa, RSA_PKCS1_PADDING) == -1) {
+
+    ERR_print_errors_cb(logError, 0);
+    OMNIORB_THROW(MARSHAL, MARSHAL_InvalidEncryptedData, CORBA::COMPLETED_NO);
+  }
+}
+
+static inline size_t
+privateDecrypt(CORBA::OctetSeq& dest, RSAKey* rsa,
+               const CORBA::Octet* data, size_t len)
+{
+  dest.length(RSA_size(rsa));
+  size_t rlen = RSA_private_decrypt(len, data, dest.NP_data(),
+                                    rsa, RSA_PKCS1_PADDING);
+  dest.length(rlen);
+  return rlen;
+}
+
+static inline size_t
+publicDecrypt(CORBA::OctetSeq& dest, RSAKey* rsa,
+              const CORBA::Octet* data, size_t len)
+{
+  dest.length(RSA_size(rsa));
+  size_t rlen = RSA_public_decrypt(len, data, dest.NP_data(),
+                                   rsa, RSA_PKCS1_PADDING);
+  dest.length(rlen);
+  return rlen;
+}
+
+static inline void
+rsaFree(RSAKey* rsa)
+{
+  RSA_free(rsa);
+}
+
+#else
+
+// Modern OpenSSL versions make direct use of RSA quite painful...
+
+static inline void
+privateEncrypt(CORBA::OctetSeq& dest, RSAKey* rsa,
+               const CORBA::Octet* data, size_t len)
+{
+  EVP_PKEY_CTX *ctx = EVP_PKEY_CTX_new(rsa, 0);
+  EVP_PKEY_sign_init(ctx);
+  EVP_PKEY_CTX_set_rsa_padding(ctx, RSA_PKCS1_PADDING);
+
+  size_t rlen;
+  EVP_PKEY_sign(ctx, 0, &rlen, data, len);
+
+  dest.length(rlen);
+  EVP_PKEY_sign(ctx, dest.NP_data(), &rlen, data, len);
+
+  EVP_PKEY_CTX_free(ctx);
+}
+
+static inline void
+publicEncrypt(CORBA::OctetSeq& dest, RSAKey* rsa,
+              const CORBA::Octet* data, size_t len)
+{
+  EVP_PKEY_CTX *ctx = EVP_PKEY_CTX_new(rsa, 0);
+  EVP_PKEY_encrypt_init(ctx);
+  EVP_PKEY_CTX_set_rsa_padding(ctx, RSA_PKCS1_PADDING);
+
+  size_t rlen;
+  EVP_PKEY_encrypt(ctx, 0, &rlen, data, len);
+
+  dest.length(rlen);
+  EVP_PKEY_encrypt(ctx, dest.NP_data(), &rlen, data, len);
+
+  EVP_PKEY_CTX_free(ctx);
+}
+
+static inline size_t
+privateDecrypt(CORBA::OctetSeq& dest, RSAKey* rsa,
+               const CORBA::Octet* data, size_t len)
+{
+  EVP_PKEY_CTX *ctx = EVP_PKEY_CTX_new(rsa, 0);
+  EVP_PKEY_decrypt_init(ctx);
+  EVP_PKEY_CTX_set_rsa_padding(ctx, RSA_PKCS1_PADDING);
+
+  size_t rlen;
+  EVP_PKEY_decrypt(ctx, 0, &rlen, data, len);
+
+  dest.length(len);
+  EVP_PKEY_decrypt(ctx, dest.NP_data(), &rlen, data, len);
+
+  EVP_PKEY_CTX_free(ctx);
+  return rlen;
+}
+
+static inline size_t
+publicDecrypt(CORBA::OctetSeq& dest, RSAKey* rsa,
+              const CORBA::Octet* data, size_t len)
+{
+  EVP_PKEY_CTX *ctx = EVP_PKEY_CTX_new(rsa, 0);
+  EVP_PKEY_verify_recover_init(ctx);
+  EVP_PKEY_CTX_set_rsa_padding(ctx, RSA_PKCS1_PADDING);
+
+  size_t rlen;
+  EVP_PKEY_verify_recover(ctx, 0, &rlen, data, len);
+
+  dest.length(len);
+  EVP_PKEY_verify_recover(ctx, dest.NP_data(), &rlen, data, len);
+
+  EVP_PKEY_CTX_free(ctx);
+  return rlen;
+}
+
+static inline void
+rsaFree(RSAKey* rsa)
+{
+  EVP_PKEY_free(rsa);
+}
+
+#endif
+
 
 const char*
 httpCrypto_AES_RSA::
@@ -325,7 +478,7 @@ writeAuthHeader(char* buf, size_t buf_space)
 
     if (pd_deadline > now) {
       // Use existing key
-      
+
       n = snprintf(buf, buf_space, "omni 1;0;%s", pd_key_ident.c_str());
 
       if (n < 0 || (size_t)n > buf_space)
@@ -336,36 +489,20 @@ writeAuthHeader(char* buf, size_t buf_space)
   }
 
   // Construct new key and send it to the server
-  int             self_size = RSA_size(pd_self_rsa);
-  int             peer_size = RSA_size(pd_peer_rsa);
-
-  CORBA::OctetSeq self_seq; self_seq.length(self_size);
-  CORBA::OctetSeq peer_seq; peer_seq.length(peer_size);
-
+  CORBA::OctetSeq self_seq;
+  CORBA::OctetSeq peer_seq;
   CORBA::Octet    key_data[KEY_SIZE * 2 + IDENT_SIZE];
-  CORBA::Octet*   self_enc = self_seq.NP_data();
-  CORBA::Octet*   peer_enc = peer_seq.NP_data();
-  
+
   if (!RAND_bytes(key_data, KEY_SIZE * 2 + IDENT_SIZE)) {
     ERR_print_errors_cb(logError, 0);
     OMNIORB_THROW(INITIALIZE, INITIALIZE_TransportError, CORBA::COMPLETED_NO);
   }
 
   // Encrypt first part of key data with my private key
-  if (RSA_private_encrypt(KEY_SIZE, key_data,
-                          self_enc, pd_self_rsa, RSA_PKCS1_PADDING) == -1) {
-
-    ERR_print_errors_cb(logError, 0);
-    OMNIORB_THROW(MARSHAL, MARSHAL_InvalidEncryptedData, CORBA::COMPLETED_NO);
-  }
+  privateEncrypt(self_seq, pd_self_rsa, key_data, KEY_SIZE);
 
   // Encrypt second part of key data with peer's public key
-  if (RSA_public_encrypt(KEY_SIZE, key_data + KEY_SIZE,
-                         peer_enc, pd_peer_rsa, RSA_PKCS1_PADDING) == -1) {
-
-    ERR_print_errors_cb(logError, 0);
-    OMNIORB_THROW(MARSHAL, MARSHAL_InvalidEncryptedData, CORBA::COMPLETED_NO);
-  }
+  publicEncrypt(peer_seq, pd_peer_rsa, key_data + KEY_SIZE, KEY_SIZE);
 
   // The AES key for the session is the XOR of the first two parts
   // of key data.
@@ -374,15 +511,15 @@ writeAuthHeader(char* buf, size_t buf_space)
     pd_key[i] = key_data[i] ^ key_data[KEY_SIZE + i];
 
   pd_key_set = 1;
-    
+
   CORBA::String_var key_ident =
     httpContext::b64encode((const char*)key_data + KEY_SIZE * 2, IDENT_SIZE);
 
   CORBA::String_var self_b64 =
-    httpContext::b64encode((const char*)self_enc, self_size);
+    httpContext::b64encode((const char*)self_seq.NP_data(), self_seq.length());
 
   CORBA::String_var peer_b64 =
-    httpContext::b64encode((const char*)peer_enc, peer_size);
+    httpContext::b64encode((const char*)peer_seq.NP_data(), peer_seq.length());
 
   n = snprintf(buf, buf_space, "omni 1;1;%s;%s;%s;%s",
                pd_self_ident.c_str(),
@@ -391,7 +528,7 @@ writeAuthHeader(char* buf, size_t buf_space)
                (const char*)peer_b64);
 
   pd_key_ident = (const char*)key_ident;
-  
+
   if (n < 0 || (size_t)n > buf_space)
     OMNIORB_THROW(MARSHAL, MARSHAL_HTTPBufferFull, CORBA::COMPLETED_NO);
 
@@ -404,7 +541,7 @@ writeAuthHeader(char* buf, size_t buf_space)
   }
   ConnectionInfo::set(ConnectionInfo::SEND_SESSION_KEY, 0,
                       pd_peer_ident.c_str());
-  
+
   return (size_t)n;
 }
 
@@ -451,7 +588,7 @@ encrypt(CORBA::Octet*       write_buf,
         CORBA::Boolean      last)
 {
   size_t written = 0;
-  
+
   if (!pd_o_cipher_ctx) {
     OMNIORB_ASSERT(pd_key_set);
 
@@ -466,10 +603,11 @@ encrypt(CORBA::Octet*       write_buf,
     if (!RAND_bytes(write_buf, IV_SIZE)) {
       ERR_print_errors_cb(logError, 0);
       OMNIORB_THROW(INITIALIZE, INITIALIZE_TransportError, CORBA::COMPLETED_NO);
-    }      
+    }
 
     // Initialise encryption
-    EVP_EncryptInit_ex(pd_o_cipher_ctx, EVP_aes_256_cbc(), 0, pd_key, write_buf);
+    EVP_EncryptInit_ex(pd_o_cipher_ctx, EVP_aes_256_cbc(), 0,
+                       pd_key, write_buf);
 
     written   += IV_SIZE;
     write_buf += IV_SIZE;
@@ -487,8 +625,8 @@ encrypt(CORBA::Octet*       write_buf,
   }
   else {
     write_size = 0;
-  }    
-    
+  }
+
   written += (size_t)write_size;
 
   if (last) {
@@ -498,7 +636,7 @@ encrypt(CORBA::Octet*       write_buf,
       ERR_print_errors_cb(logError, 0);
       OMNIORB_THROW(MARSHAL, MARSHAL_InvalidEncryptedData, CORBA::COMPLETED_NO);
     }
-    
+
     written += (size_t)write_size;
 
     EVP_CIPHER_CTX_free(pd_o_cipher_ctx);
@@ -538,7 +676,7 @@ decrypt(CORBA::Octet*       write_buf,
       OMNIORB_THROW(MARSHAL, MARSHAL_InvalidEncryptedData, CORBA::COMPLETED_NO);
 
     // The message starts with the IV.
-    
+
     if (!EVP_DecryptInit_ex(pd_i_cipher_ctx, EVP_aes_256_cbc(), 0,
                             pd_key, read_buf)) {
       ERR_print_errors_cb(logError, 0);
@@ -562,7 +700,7 @@ decrypt(CORBA::Octet*       write_buf,
   else {
     write_size = 0;
   }
-    
+
   size_t written = (size_t)write_size;
 
   if (last) {
@@ -572,13 +710,13 @@ decrypt(CORBA::Octet*       write_buf,
       ERR_print_errors_cb(logError, 0);
       OMNIORB_THROW(MARSHAL, MARSHAL_InvalidEncryptedData, CORBA::COMPLETED_NO);
     }
-      
+
     written += (size_t)write_size;
-    
+
     EVP_CIPHER_CTX_free(pd_i_cipher_ctx);
     pd_i_cipher_ctx = 0;
   }
-  
+
   return written;
 }
 
@@ -684,13 +822,13 @@ httpCryptoManager_AES_RSA_impl::
   RSAKeyMap::iterator it;
 
   for (it = pd_clients.begin(); it != pd_clients.end(); ++it)
-    RSA_free(it->second);
+    rsaFree(it->second);
 
   for (it = pd_servers.begin(); it != pd_servers.end(); ++it)
-    RSA_free(it->second);
+    rsaFree(it->second);
 
   if (pd_self_rsa) {
-    RSA_free(pd_self_rsa);
+    rsaFree(pd_self_rsa);
     pd_self_rsa = 0;
   }
 }
@@ -704,7 +842,7 @@ scavenge()
 
   omni_time_t now;
   omni_thread::get_time(now);
-  
+
   AESKeyMap::iterator it, next;
 
   for (it = pd_session_keys.begin(); it != pd_session_keys.end(); it = next) {
@@ -732,14 +870,14 @@ init(const char*    ident,
   omni_tracedmutex_lock l(pd_lock);
 
   pd_self_ident = ident;
-  
+
   if (pd_self_rsa) {
-    RSA_free(pd_self_rsa);
+    rsaFree(pd_self_rsa);
     pd_self_rsa = 0;
   }
 
   BIO* bio;
-  
+
   if (is_filename) {
     bio = BIO_new_file(private_key, "r");
     if (!bio) {
@@ -754,9 +892,14 @@ init(const char*    ident,
     bio = BIO_new_mem_buf(private_key, -1);
   }
 
+#ifdef USE_OLD_RSA_FUNCTIONS
   pd_self_rsa = PEM_read_bio_RSAPrivateKey(bio, 0, 0, 0);
+#else
+  pd_self_rsa = PEM_read_bio_PrivateKey(bio, &pd_self_rsa, 0, 0);
+#endif
+
   BIO_free(bio);
-  
+
   if (!pd_self_rsa) {
     ERR_print_errors_cb(logError, 0);
     OMNIORB_THROW(INITIALIZE, INITIALIZE_TransportError, CORBA::COMPLETED_NO);
@@ -772,12 +915,12 @@ init(const char*    ident,
 
 
 
-static RSA*
+static RSAKey*
 readPublicKey(const char*    public_key,
               CORBA::Boolean is_filename)
 {
   BIO* bio;
-  
+
   if (is_filename) {
     bio = BIO_new_file(public_key, "r");
     if (!bio) {
@@ -792,7 +935,14 @@ readPublicKey(const char*    public_key,
     bio = BIO_new_mem_buf(public_key, -1);
   }
 
-  RSA* rsa = PEM_read_bio_RSA_PUBKEY(bio, 0, 0, 0);
+  RSAKey* rsa = 0;
+
+#ifdef USE_OLD_RSA_FUNCTIONS
+  rsa = PEM_read_bio_RSA_PUBKEY(bio, 0, 0, 0);
+#else
+  rsa = PEM_read_bio_PUBKEY(bio, &rsa, 0, 0);
+#endif
+
   BIO_free(bio);
 
   if (!rsa) {
@@ -809,18 +959,18 @@ addClient(const char*    ident,
           const char*    public_key,
           CORBA::Boolean is_filename)
 {
-  RSA* rsa = readPublicKey(public_key, is_filename);
+  RSAKey* rsa = readPublicKey(public_key, is_filename);
 
   omni_tracedmutex_lock l(pd_lock);
 
   if (!pd_initialised) {
-    RSA_free(rsa);
+    rsaFree(rsa);
     OMNIORB_THROW(INITIALIZE, INITIALIZE_TransportError, CORBA::COMPLETED_NO);
   }
-  
-  RSA*& entry = pd_clients[ident];
+
+  RSAKey*& entry = pd_clients[ident];
   if (entry)
-    RSA_free(entry);
+    rsaFree(entry);
 
   entry = rsa;
 }
@@ -835,7 +985,7 @@ removeClient(const char* ident)
   RSAKeyMap::iterator it = pd_clients.find(ident);
 
   if (it != pd_clients.end()) {
-    RSA_free(it->second);
+    rsaFree(it->second);
     pd_clients.erase(it);
     return 1;
   }
@@ -849,18 +999,18 @@ addServer(const char*    url,
           const char*    public_key,
           CORBA::Boolean is_filename)
 {
-  RSA* rsa = readPublicKey(public_key, is_filename);
+  RSAKey* rsa = readPublicKey(public_key, is_filename);
 
   omni_tracedmutex_lock l(pd_lock);
 
   if (!pd_initialised) {
-    RSA_free(rsa);
+    rsaFree(rsa);
     OMNIORB_THROW(INITIALIZE, INITIALIZE_TransportError, CORBA::COMPLETED_NO);
   }
-  
-  RSA*& entry = pd_servers[url];
+
+  RSAKey*& entry = pd_servers[url];
   if (entry)
-    RSA_free(entry);
+    rsaFree(entry);
 
   entry = rsa;
 }
@@ -875,7 +1025,7 @@ removeServer(const char* url)
   RSAKeyMap::iterator it = pd_servers.find(url);
 
   if (it != pd_servers.end()) {
-    RSA_free(it->second);
+    rsaFree(it->second);
     pd_servers.erase(it);
     return 1;
   }
@@ -898,7 +1048,7 @@ cryptoForServer(const char* url, CORBA::Boolean new_key)
   if (rit == pd_servers.end())
     return 0;
 
-  RSA* server_rsa = rit->second;
+  RSAKey* server_rsa = rit->second;
 
   if (new_key) {
     pd_session_keys.erase(url_string);
@@ -911,7 +1061,7 @@ cryptoForServer(const char* url, CORBA::Boolean new_key)
       omni_thread::get_time(now);
 
       SessionKey& sk = it->second;
-      
+
       if (sk.deadline > now) {
         // Use existing key
         return new httpCrypto_AES_RSA(this, (const CORBA::Octet*)sk.key.data(),
@@ -924,7 +1074,7 @@ cryptoForServer(const char* url, CORBA::Boolean new_key)
   // Generate a new key in writeAuthHeader. Note that we permit
   // multiple threads to decide to do this concurrently because that
   // is more efficient and simpler than trying to synchronise them.
-  
+
   return new httpCrypto_AES_RSA(this, url_string, pd_self_ident,
                                 pd_self_rsa, server_rsa);
 }
@@ -965,7 +1115,7 @@ readAuthHeader(const char* host, const char* auth)
       omni_thread::get_time(now);
 
       const SessionKey& sk = sit->second;
-      
+
       if (sk.deadline > now) {
         return new httpCrypto_AES_RSA(this, (const CORBA::Octet*)sk.key.data(),
                                       sk.key_ident, sk.peer_ident, sk.deadline);
@@ -982,7 +1132,7 @@ readAuthHeader(const char* host, const char* auth)
     std::string       client_ident, key_ident, client_b64, self_b64;
     std::string       data(auth+9);
     std::stringstream ss(data);
-    
+
     std::getline(ss, client_ident, ';');
     std::getline(ss, key_ident,    ';');
     std::getline(ss, client_b64,   ';');
@@ -990,7 +1140,7 @@ readAuthHeader(const char* host, const char* auth)
 
     if (!ss || !ss.eof())
       OMNIORB_THROW(MARSHAL, MARSHAL_HTTPHeaderInvalid, CORBA::COMPLETED_NO);
-    
+
     RSAKeyMap::iterator cit = pd_clients.find(client_ident);
 
     if (cit == pd_clients.end()) {
@@ -1007,40 +1157,38 @@ readAuthHeader(const char* host, const char* auth)
                     CORBA::COMPLETED_NO);
     }
 
-    RSA*                client_rsa = cit->second;
+    RSAKey*         client_rsa = cit->second;
+    CORBA::OctetSeq client_key;
+    CORBA::OctetSeq self_key;
+
     size_t              client_len, self_len;
     CORBA::String_var   client_enc_s, self_enc_s;
     const CORBA::Octet* client_enc;
     const CORBA::Octet* self_enc;
-    
+
     client_enc_s = httpContext::b64decode(client_b64.c_str(), client_len);
     client_enc   = (const CORBA::Octet*)(const char*)client_enc_s;
 
     self_enc_s   = httpContext::b64decode(self_b64.c_str(),   self_len);
     self_enc     = (const CORBA::Octet*)(const char*)self_enc_s;
 
-    CORBA::OctetSeq key_seq;
-    key_seq.length(RSA_size(client_rsa) + RSA_size(pd_self_rsa));
-    
-    CORBA::Octet*   key_data = key_seq.NP_data();
+    if (publicDecrypt(client_key, client_rsa,
+                      client_enc, client_len) != KEY_SIZE) {
 
-    if (RSA_public_decrypt(client_len, client_enc, key_data,
-                           client_rsa, RSA_PKCS1_PADDING) != KEY_SIZE) {
-      
       OMNIORB_THROW(MARSHAL, MARSHAL_InvalidEncryptedData, CORBA::COMPLETED_NO);
     }
 
-    if (RSA_private_decrypt(self_len, self_enc, key_data + KEY_SIZE,
-                            pd_self_rsa, RSA_PKCS1_PADDING) != KEY_SIZE) {
-      
+    if (privateDecrypt(self_key, pd_self_rsa,
+                       self_enc, self_len) != KEY_SIZE) {
+
       OMNIORB_THROW(MARSHAL, MARSHAL_InvalidEncryptedData, CORBA::COMPLETED_NO);
     }
 
     AESKey key;
 
     for (size_t i=0; i != KEY_SIZE; ++i)
-      key[i] = key_data[i] ^ key_data[KEY_SIZE + i];
-    
+      key[i] = client_key[i] ^ self_key[i];
+
     SessionKey& sk = pd_session_keys[key_ident];
 
     sk.peer_ident = client_ident;
@@ -1055,7 +1203,7 @@ readAuthHeader(const char* host, const char* auth)
     ConnectionInfo::set(ConnectionInfo::RECEIVED_SESSION_KEY, 0,
                         client_ident.c_str());
 
-    
+
     return new httpCrypto_AES_RSA(this, key, key_ident, client_ident,
                                   sk.deadline);
   }
