@@ -309,31 +309,65 @@ CORBA::ORB_init(int& argc, char** argv, const char* orb_identifier,
     // Call attach method of each initialiser object.
     // The order of these calls must take into account of the dependency
     // among the modules.
-    omni_giopEndpoint_initialiser_.attach();
-    omni_transportRules_initialiser_.attach();
-    omni_interceptor_initialiser_.attach();
-    omni_omniInternal_initialiser_.attach();
-    omni_corbaOrb_initialiser_.attach();
-    omni_objadpt_initialiser_.attach();
-    omni_giopStreamImpl_initialiser_.attach();
-    omni_omniIOR_initialiser_.attach();
-    omni_ior_initialiser_.attach();
-    omni_codeSet_initialiser_.attach();
-    omni_cdrStream_initialiser_.attach();
-    omni_omniTransport_initialiser_.attach();
-    omni_giopRope_initialiser_.attach();
-    omni_giopserver_initialiser_.attach();
-    omni_giopbidir_initialiser_.attach();
-    omni_giopStrand_initialiser_.attach();
-    omni_omniCurrent_initialiser_.attach();
-    omni_dynamiclib_initialiser_.attach();
-    omni_ObjRef_initialiser_.attach();
-    omni_initRefs_initialiser_.attach();
-    omni_orbOptions_initialiser_.attach();
-    omni_poa_initialiser_.attach();
-    omni_uri_initialiser_.attach();
-    omni_invoker_initialiser_.attach();
-    omni_hooked_initialiser_.attach();
+    int i=0;
+
+    try {
+      omni_giopEndpoint_initialiser_.attach();    i++;
+      omni_transportRules_initialiser_.attach();  i++;
+      omni_interceptor_initialiser_.attach();     i++;
+      omni_omniInternal_initialiser_.attach();    i++;
+      omni_corbaOrb_initialiser_.attach();        i++;
+      omni_objadpt_initialiser_.attach();         i++;
+      omni_giopStreamImpl_initialiser_.attach();  i++;
+      omni_omniIOR_initialiser_.attach();         i++;
+      omni_ior_initialiser_.attach();             i++;
+      omni_codeSet_initialiser_.attach();         i++;
+      omni_cdrStream_initialiser_.attach();       i++;
+      omni_omniTransport_initialiser_.attach();   i++;
+      omni_giopRope_initialiser_.attach();        i++;
+      omni_giopserver_initialiser_.attach();      i++;
+      omni_giopbidir_initialiser_.attach();       i++;
+      omni_giopStrand_initialiser_.attach();      i++;
+      omni_omniCurrent_initialiser_.attach();     i++;
+      omni_dynamiclib_initialiser_.attach();      i++;
+      omni_ObjRef_initialiser_.attach();          i++;
+      omni_initRefs_initialiser_.attach();        i++;
+      omni_orbOptions_initialiser_.attach();      i++;
+      omni_poa_initialiser_.attach();             i++;
+      omni_uri_initialiser_.attach();             i++;
+      omni_invoker_initialiser_.attach();         i++;
+      omni_hooked_initialiser_.attach();          i++;
+    }
+    catch (...) {
+      switch (i) {
+      case 25: omni_hooked_initialiser_.detach();
+      case 24: omni_invoker_initialiser_.detach();
+      case 23: omni_uri_initialiser_.detach();
+      case 22: omni_poa_initialiser_.detach();
+      case 21: omni_orbOptions_initialiser_.detach();
+      case 20: omni_initRefs_initialiser_.detach();
+      case 19: omni_ObjRef_initialiser_.detach();
+      case 18: omni_dynamiclib_initialiser_.detach();
+      case 17: omni_omniCurrent_initialiser_.detach();
+      case 16: omni_giopStrand_initialiser_.detach();
+      case 15: omni_giopbidir_initialiser_.detach();
+      case 14: omni_giopserver_initialiser_.detach();
+      case 13: omni_giopRope_initialiser_.detach();
+      case 12: omni_omniTransport_initialiser_.detach();
+      case 11: omni_cdrStream_initialiser_.detach();
+      case 10: omni_codeSet_initialiser_.detach();
+      case  9: omni_ior_initialiser_.detach();
+      case  8: omni_omniIOR_initialiser_.detach();
+      case  7: omni_giopStreamImpl_initialiser_.detach();
+      case  6: omni_objadpt_initialiser_.detach();
+      case  5: omni_corbaOrb_initialiser_.detach();
+      case  4: omni_omniInternal_initialiser_.detach();
+      case  3: omni_interceptor_initialiser_.detach();
+      case  2: omni_transportRules_initialiser_.detach();
+      case  1: omni_giopEndpoint_initialiser_.detach();
+      };
+      throw;
+    }
 
     if (orbParameters::lcdMode) {
       enableLcdMode();
@@ -849,11 +883,25 @@ static omnivector<omniInitialiser*>*& the_hooked_list()
 class omni_hooked_initialiser : public omniInitialiser {
 public:
   void attach() {
-    omnivector<omniInitialiser*>::iterator i    = the_hooked_list()->begin();
-    omnivector<omniInitialiser*>::iterator last = the_hooked_list()->end();
+    omnivector<omniInitialiser*>::iterator first = the_hooked_list()->begin();
+    omnivector<omniInitialiser*>::iterator last  = the_hooked_list()->end();
+    omnivector<omniInitialiser*>::iterator i;
 
-    for (; i != last; i++) {
-      (*i)->attach();
+    try {
+      for (i=first; i != last; i++) {
+        (*i)->attach();
+      }
+    }
+    catch (...) {
+      if (i != first) {
+        while (1) {
+          --i;
+          (*i)->detach();
+          if (i == first)
+            break;
+        }
+      }
+      throw;
     }
   }
 
