@@ -625,9 +625,8 @@ public:
 "                          * may not be supported on the platform.\n") {}
 
 
-  void visit(const char* value,orbOptions::Source)
-    OMNI_THROW_SPEC (orbOptions::BadParam)
-  {
+  void visit(const char* value,orbOptions::Source) {
+
     omniObjAdapter::Options::EndpointURI* opt;
     opt = new omniObjAdapter::Options::EndpointURI();
     opt->no_publish = 0;
@@ -668,9 +667,8 @@ public:
 			"-ORBendPointNoPublish <endpoint uri>") {}
 
 
-  void visit(const char* value,orbOptions::Source)
-    OMNI_THROW_SPEC (orbOptions::BadParam)
-  {
+  void visit(const char* value,orbOptions::Source) {
+
     omniObjAdapter::Options::EndpointURI* opt;
     opt = new omniObjAdapter::Options::EndpointURI();
     opt->no_publish = 1;
@@ -705,9 +703,7 @@ public:
 			"-ORBendPointPublish <publish options>") {}
 
 
-  void visit(const char* value,orbOptions::Source)
-    OMNI_THROW_SPEC (orbOptions::BadParam)
-  {
+  void visit(const char* value,orbOptions::Source) {
     omniObjAdapter::options.publish = value;
   }
 
@@ -730,9 +726,8 @@ public:
 			"-ORBendPointNoListen <endpoint uri>") {}
 
 
-  void visit(const char* value,orbOptions::Source)
-    OMNI_THROW_SPEC (orbOptions::BadParam)
-  {
+  void visit(const char* value,orbOptions::Source) {
+
     omniObjAdapter::Options::EndpointURI* opt;
     opt = new omniObjAdapter::Options::EndpointURI();
     opt->no_publish = 0;
@@ -764,9 +759,7 @@ public:
 			"-ORBendPointPublishAllIFs < 0 | 1 >") {}
 
 
-  void visit(const char* value,orbOptions::Source)
-    OMNI_THROW_SPEC (orbOptions::BadParam)
-  {
+  void visit(const char* value,orbOptions::Source) {
     CORBA::Boolean v;
     if (!orbOptions::getBoolean(value,v)) {
       throw orbOptions::BadParam(key(),value,
