@@ -167,7 +167,12 @@ sslAddress::Connect(const omni_time_t& deadline,
 	  return 0;
 	}
         ConnectionInfo::set(ConnectionInfo::TLS_CONNECTED, 0, addr_str);
-	return new sslActiveConnection(sock,ssl);
+
+        CORBA::Boolean concurrent =
+          (!orbParameters::oneCallPerConnection ||
+           orbParameters::offerBiDirectionalGIOP);
+
+        return new sslActiveConnection(sock, ssl, concurrent);
       }
 
     case SSL_ERROR_WANT_READ:
