@@ -325,7 +325,12 @@ sslEndpoint::AcceptAndMonitor(giopConnection::notifyReadable_t func,
       SSL_set_fd(ssl, pd_new_conn_socket);
       SSL_set_accept_state(ssl);
 
-      sslConnection* nc = new sslConnection(pd_new_conn_socket, ssl, this);
+      CORBA::Boolean concurrent =
+        (orbParameters::maxServerThreadPerConnection > 1 ||
+         orbParameters::acceptBiDirectionalGIOP);
+      
+      sslConnection* nc = new sslConnection(pd_new_conn_socket, ssl, this,
+                                            concurrent);
       ConnectionInfo::set(ConnectionInfo::ACCEPTED_CONNECTION, 0,
                           nc->peeraddress());
       return nc;
