@@ -35,8 +35,17 @@
 #include <GIOP_S.h>
 #include <transportRules.h>
 #include <poaimpl.h>
-#include <zlib.h>
-#include "zlibCompressor.h"
+
+#ifdef OMNI_ENABLE_ZIOP_ZSTD
+#  include <zstd.h>
+#  include "zstdCompressor.h"
+#endif
+
+#ifdef OMNI_ENABLE_ZIOP_ZLIB
+#  include <zlib.h>
+#  include "zlibCompressor.h"
+#endif
+
 
 OMNI_USING_NAMESPACE(omni)
 
@@ -276,10 +285,22 @@ setGlobalPolicies(const CORBA::PolicyList& policies)
   // Set defaults
   g_compression_enabled = 1;
 
-  g_compressor_ids.length(1);
-  g_compressor_ids[0].compressor_id     = Compression::COMPRESSORID_ZLIB;
-  g_compressor_ids[0].compression_level = 6;
+  CORBA::ULong idx = 0;
 
+#ifdef OMNI_ENABLE_ZIOP_ZSTD
+  g_compressor_ids.length(idx+1);
+  g_compressor_ids[idx].compressor_id     = Compression::COMPRESSORID_OMNI_ZSTD;
+  g_compressor_ids[idx].compression_level = 3;
+  ++idx;
+#endif
+  
+#ifdef OMNI_ENABLE_ZIOP_ZLIB
+  g_compressor_ids.length(idx+1);
+  g_compressor_ids[idx].compressor_id     = Compression::COMPRESSORID_ZLIB;
+  g_compressor_ids[idx].compression_level = 6;
+  ++idx;
+#endif
+  
   decodePolicies(policies,
                  g_compression_enabled,
                  g_compressor_ids,
@@ -536,8 +557,17 @@ giopCompressorImpl::compressBuffer(giopStream*        stream,
                  g_buf_start[3] == 'P');
 
   // Allocate new buffer
-  CORBA::ULong       req_len = GIOP_ZIOP_HEADER + compressBound(g_buf_len);
-  giopStream_Buffer* z_buf   = giopStream_Buffer::newBuffer(
+  CORBA::ULong req_len;
+
+#ifdef OMNI_ENABLE_ZIOP_ZSTD
+  // zstd's worst case is worse than zlib's, so we use that to choose
+  // the buffer size.
+  req_len = GIOP_ZIOP_HEADER + ZSTD_COMPRESSBOUND(g_buf_len);
+#else
+  req_len = GIOP_ZIOP_HEADER + compressBound(g_buf_len);
+#endif
+
+  giopStream_Buffer* z_buf = giopStream_Buffer::newBuffer(
     req_len <= giopStream::bufferSize ?
     giopStream::bufferSize : req_len);
 
@@ -1125,8 +1155,19 @@ public:
   void attach() {
     the_manager = new omniCompressionManager();
     
-    Compression::CompressorFactory_var zf = new zlibCompressorFactory();
+    Compression::CompressorFactory_var zf;
+
+#ifdef OMNI_ENABLE_ZIOP_ZSTD
+    omniORB::logs(10, "Register zstd compressor.");
+    zf = new zstdCompressorFactory();
     the_manager->register_factory(zf);
+#endif
+
+#ifdef OMNI_ENABLE_ZIOP_ZLIB
+    omniORB::logs(10, "Register zlib compressor.");
+    zf = new zlibCompressorFactory();
+    the_manager->register_factory(zf);
+#endif
 
     giopStream::compressorFactory = new giopCompressorFactoryImpl();
 
